@@ -23,6 +23,7 @@ from sklearn.preprocessing import normalize
 import random
 from collections import Counter
 import scipy
+import pygam as pg
 from scipy.sparse.csgraph import minimum_spanning_tree, connected_components
 from matplotlib.animation import FuncAnimation, writers
 
@@ -30,6 +31,54 @@ def func_mode(ll):
     # return MODE of list ll
     # If multiple items are maximal, the function returns the first one encountered.
     return max(set(ll), key=ll.count)
+def get_gene_trend(via_object, marker_lineages:list=[], df_gene_exp:pd.DataFrame=None,n_splines:int=10, spline_order:int=4):
+    '''
+    Get the gene trend vs pseudotime for a lineage (terminal cell fate)
+    :param via_object:
+    :param marker_lineages:
+    :param df_gene_exp:
+    :param n_splines:
+    :param spline_order:
+    :return: dict of dicts. First dict keys corresponding to terminal cluster of a lineage, second dict having keys "trends": entries with pandas DataFrame with genes (rows) x ("pseudotime") for that lineage and "name" : majority true label
+    '''
+    sc_pt = via_object.single_cell_pt_markov
+    sc_bp = via_object.single_cell_bp
+    n_terminal_states = sc_bp.shape[1]
+    ts = via_object.terminal_clusters
+    trends_dict = {}
+    for i_terminal in range(n_terminal_states):
+        df_trends = pd.DataFrame()
+        #print('ts[i_terminal]',ts[i_terminal],len(np.where(sc_bp[:, i_terminal] > 0.9)[0]))
+        if (ts[i_terminal] in marker_lineages and len(np.where(sc_bp[:, i_terminal] > 0.9)[0]) > 0):  # check if terminal state is in marker_lineage and i_terminal can be reached
+            loc_i = np.where(sc_bp[:, i_terminal] > 0.9)[0]
+            val_pt = [sc_pt[pt_i] for pt_i in loc_i]  # TODO,  replace with array to speed up
+
+            max_val_pt = max(val_pt)
+
+            loc_i_bp = np.where(sc_bp[:, i_terminal] > 0.000)[0]  # 0.001
+            loc_i_sc = np.where(np.asarray(sc_pt) <= max_val_pt)[0]
+
+            loc_ = np.intersect1d(loc_i_bp, loc_i_sc)
+            if len(loc_) > 1:
+                gam_in = np.asarray(sc_pt)[loc_]
+                x = gam_in.reshape(-1, 1)
+                for gene_i in df_gene_exp.columns:
+                    y = np.asarray(df_gene_exp[gene_i])[loc_].reshape(-1, 1)
+
+                    weights = np.asarray(sc_bp[:, i_terminal])[loc_].reshape(-1, 1)
+
+                    geneGAM = pg.LinearGAM(n_splines=n_splines, spline_order=spline_order, lam=10).fit(x, y, weights=weights)
+                    xval = np.linspace(min(sc_pt), max_val_pt, 100 * 2)
+                    yg = geneGAM.predict(X=xval)
+                    df_trends[str(gene_i)] = yg
+
+            else:
+                print(f'{datetime.now()}\tLineage {i_terminal} cannot be reached. Exclude this lineage in trend plotting')
+            loc_labels = np.where(np.asarray(via_object.labels)==ts[i_terminal])[0]
+            majority_composition = func_mode(list(np.asarray(via_object.true_label)[loc_labels]))
+            trends_dict[ts[i_terminal]]={'trends':df_trends.transpose(), 'name': majority_composition}
+    return trends_dict
+
 
 def csr_mst(adjacency):
     # return minimum spanning tree from adjacency matrix (csr)
@@ -256,7 +305,7 @@ def affinity_milestone_knn(data, knn_struct,k:int=10, time_series_labels:list=[]
                 if abs(time_series_labels[n_i] - t_row)>t_diff_mean*t_diff_step:
                     count= count+1
                     if np.sum(msk[row[0]])>4: msk[row[0],e_i]=False # we want to ensure that each cell has at least 5 nn
-        print('number of non temporal neighbors removed', count)
+        print(f'{datetime.now()}\tNumber of non temporal neighbors removed', count)
     else:
         msk = np.full_like(distance_array, True, dtype=np.bool_)
         # print('all edges', np.sum(msk))
@@ -269,7 +318,7 @@ def affinity_milestone_knn(data, knn_struct,k:int=10, time_series_labels:list=[]
     row_znormed_dist_array = np.nan_to_num(row_znormed_dist_array, copy=True, nan=1, posinf=1, neginf=1)
     row_znormed_dist_array[row_znormed_dist_array > 10] = 0
     affinity_array = np.exp(-row_znormed_dist_array)
-    print(affinity_array.shape, 'affinity shape', affinity_array[0:5,:])
+
     n_neighbors = neighbor_array.shape[1]
     n_cells = neighbor_array.shape[0]
 
@@ -295,30 +344,30 @@ def affinity_milestone_knn(data, knn_struct,k:int=10, time_series_labels:list=[]
     return result
 
 
-def sgd_mds(via_graph: csr_matrix, X_pca, t_diff_op: int = 1, ndims: int = 2, random_seed = 0):
+def sgd_mds(via_graph: csr_matrix, X_pca, diff_op: int = 1, ndims: int = 2, random_seed = 0, double_diffusion:bool=True):
     '''
 
     :param via_graph: via_graph = v0.csr_full_graph #single cell knn graph representation based on hnsw
-    :param t_diff_op:
+    :param diff_op: power of diffusion operation
     :param ndims:
     :return:
     '''
     # outlier handling of graph edges
     via_graph.data = np.clip(via_graph.data, np.percentile(via_graph.data, 10), np.percentile(via_graph.data, 90))
     row_stoch = normalize(via_graph, norm='l1', axis=1)
-    row_stoch = row_stoch ** t_diff_op
+    row_stoch = row_stoch ** diff_op
     #msk = row_stoch==0
     from scipy.sparse.csgraph import connected_components
     n_components, labels_cc = connected_components(csgraph=row_stoch, directed=False, return_labels=True)
-    if n_components>1: print('Please rerun with higher knn value. disconnected components exist')
+    if n_components>1: print('Considering re-running with higher knn value. disconnected components exist')
 
     temp_pca = csr_matrix(X_pca)
 
     X_mds = row_stoch * temp_pca  # matrix multiplication to diffuse the pcs
 
-    print('doing pdist')
+
     X_mds = squareform(pdist(X_mds.todense()))
-    print('shape of squareform', X_mds.shape)
+
     #print(X_mds[0:10,:])
     #print(X_mds[490:499,:])
     #X_mds[msk.todense()]=np.amax(X_mds)
@@ -332,7 +381,7 @@ def sgd_mds(via_graph: csr_matrix, X_pca, t_diff_op: int = 1, ndims: int = 2, ra
     Y_classic = classic(X_mds, n_components=ndims, random_state=random_seed)
 
     X_mds = sgd(X_mds, n_components=ndims, random_state=random_seed, init=Y_classic)
-
+    if double_diffusion==True: X_mds = row_stoch*X_mds #added Dec 12 10pm to test diffusion of x-mds after mds
     return X_mds
 
 
@@ -614,7 +663,7 @@ def sc_loc_ofsuperCluster_PCAspace(p0, p1, idx):
     # print('new_superclust_index_ds',new_superclust_index_ds)
     return new_superclust_index_ds
 
-def plot_sc_pb(ax, fig, embedding, prob, ti, cmap_name: str ='plasma', scatter_size=None):
+def plot_sc_pb(ax, fig, embedding, prob, ti, cmap_name: str ='plasma', scatter_size=None, vmax=99, fontsize:int=10):
     '''
     This is a helper function called by draw_sc_lineage_probability which plots the single-cell lineage probabilities
 
@@ -628,33 +677,45 @@ def plot_sc_pb(ax, fig, embedding, prob, ti, cmap_name: str ='plasma', scatter_s
     :return:
     '''
     from mpl_toolkits.axes_grid1 import make_axes_locatable
-    prob = np.sqrt(prob)  # scale values to improve visualization of colors
+    #prob = np.sqrt(prob)  # scale values to improve visualization of colors
+    vmax = np.percentile(prob,vmax)
+    #vmax=1
     cmap = matplotlib.cm.get_cmap(cmap_name)
-    norm = matplotlib.colors.Normalize(vmin=0, vmax=np.max(prob))
+    #norm = matplotlib.colors.Normalize(vmin=0, vmax=np.max(prob))
     if scatter_size is None:
         size_point = 10 if embedding.shape[0] > 10000 else 30
     else: size_point = scatter_size
     # changing the alpha transparency parameter for plotting points
-    im =ax.scatter(embedding[:, 0], embedding[:, 1], c=prob, s=0.01, cmap=cmap_name,    edgecolors = 'none')
-    ax.set_title('Lineage: ' + str(ti))
+
+    c = cmap(prob).reshape(-1, 4)
+    im =ax.scatter(embedding[:, 0], embedding[:, 1], c=prob, s=0.01, cmap=cmap_name,    edgecolors = 'none',vmin=0, vmax=vmax) #prevent auto-normalization of colors
+    #im = ax.scatter(embedding[:, 0], embedding[:, 1], c=c, s=0.01,  edgecolors='none')
+    ax.set_title('Lineage: ' + str(ti), fontsize=int(fontsize*1.2))
+
     divider = make_axes_locatable(ax)
     cax = divider.append_axes('right', size='5%', pad=0.05)
-    fig.colorbar(im, cax=cax, orientation='vertical', label='lineage likelihood')
+    cb = fig.colorbar(im, cax=cax, orientation='vertical', label='lineage likelihood')
 
-    c = cmap(norm(prob)).reshape(-1, 4)
+    ax_cb = cb.ax
+    text = ax_cb.yaxis.label
+    font = matplotlib.font_manager.FontProperties( size=fontsize)#family='times new roman', style='italic',
+    text.set_font_properties(font)
+    ax_cb.tick_params(labelsize=int(fontsize*0.8))
+    c = cmap(prob).reshape(-1, 4)
+
+    #c = cmap(norm(prob)).reshape(-1, 4)
     loc_c = np.where(prob <= 0.3)[0]
-    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none',alpha=0.2, cmap=cmap_name)
+    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none',alpha=0.2, cmap=cmap_name,vmin=0, vmax=vmax)
     c[loc_c, 3] = 0.2
     loc_c = np.where((prob > 0.3) & (prob <= 0.5))[0]
     c[loc_c, 3] = 0.5
-    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.5, cmap=cmap_name)
+    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.5, cmap=cmap_name,vmin=0, vmax=vmax)
     loc_c = np.where((prob > 0.5) & (prob <= 0.7))[0]
     c[loc_c, 3] = 0.8
-    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.8, cmap=cmap_name)
+    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.8, cmap=cmap_name,vmin=0, vmax=vmax)
     loc_c = np.where((prob > 0.7))[0]
     c[loc_c, 3] = 0.8
-    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.8, cmap=cmap_name)
-
+    ax.scatter(embedding[loc_c, 0], embedding[loc_c, 1], c=prob[loc_c], s=size_point, edgecolors='none', alpha=0.8, cmap=cmap_name,vmin=0, vmax=vmax)
 
 from datashader.bundling import connect_edges, hammer_bundle
 def sigmoid_func(X):
@@ -683,7 +744,7 @@ def cosine_sim(A,B):
     '''
     #numerator
     num = np.dot(A, B.T)
-    #print('num',num)
+
     #denominator
     p1 = np.sqrt(np.sum(A ** 2, axis=1))#[:, np.newaxis] when A and B are both matrices
     p2 = np.sqrt(np.sum(B ** 2))
@@ -748,7 +809,7 @@ def make_edgebundle_viagraph(layout=None, graph=None,initial_bandwidth = 0.05, d
     return hb
 
 
-def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=None, sc_pt:list = None, initial_bandwidth=0.03, decay=0.7, n_milestones:int=None, milestone_labels:list=[], sc_labels_numeric:list=None, weighted=True, global_visual_pruning=0.5):
+def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=None, sc_pt:list = None, initial_bandwidth=0.03, decay=0.7, n_milestones:int=None, milestone_labels:list=[], sc_labels_numeric:list=None, weighted:bool=True, global_visual_pruning:float=0.5):
     '''
     # Perform Edgebundling of edges in a milestone level to return a hammer bundle of milestone-level edges. This is more granular than the original parc-clusters but less granular than single-cell level and hence also less computationally expensive
     # requires some type of embedding (n_samples x 2) to be available
@@ -758,10 +819,10 @@ def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=
     :param sc_graph: igraph graph set as the via attribute self.ig_full_graph (affinity graph)
     :param initial_bandwidth: increasing bw increases merging of minor edges
     :param decay: increasing decay increases merging of minor edges #https://datashader.org/user_guide/Networks.html
-    :param milestone_labels:list=[] single-cell level labels (clusters, groups, which function as milestone groupings of the single cells)
+    :param milestone_labels: default list=[]. Usually autocomputed. but can provide as single-cell level labels (clusters, groups, which function as milestone groupings of the single cells)
     :param sc_labels_numeric:list=None (default) automatically selects the sequential numeric time-series values in
     :return: dictionary containing keys: hb_dict['hammerbundle'] = hb hammerbundle class with hb.x and hb.y containing the coords
-                hb_dict['milestone_embedding'] dataframe with 'x' and 'y' columns for each milestone and hb_dict['edges'] dataframe with columns ['source','target'] milestone for each each and ['cluster_pop']
+                hb_dict['milestone_embedding'] dataframe with 'x' and 'y' columns for each milestone and hb_dict['edges'] dataframe with columns ['source','target'] milestone for each each and ['cluster_pop'], hb_dict['sc_milestone_labels'] is a list of milestone label for each single cell
 
     '''
     if embedding is None:
@@ -771,20 +832,21 @@ def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=
         if via_object is not None: sc_graph =via_object.ig_full_graph
     if embedding is None: print(f'{datetime.now()}\tERROR: Please provide either an embedding (single cell level) or via_object with an embedding attribute!')
     n_samples = embedding.shape[0]
-    if n_milestones is None: n_milestones = max(100, int(0.01*n_samples))
+    if n_milestones is None:
+        n_milestones = min(n_samples,max(100, int(0.01*n_samples)))
     #milestone_indices = random.sample(range(n_samples), n_milestones)  # this is sampling without replacement
     if len(milestone_labels)==0:
-        print(f'{datetime.now()}\tStart kmeans milestone')
+        print(f'{datetime.now()}\tStart finding milestones')
         from sklearn.cluster import KMeans
         kmeans = KMeans(n_clusters=n_milestones, random_state=1).fit(embedding)
         milestone_labels = kmeans.labels_.flatten().tolist()
-        print(f'{datetime.now()}\tEnd kmeans milestone')
+        print(f'{datetime.now()}\tEnd milestones')
         #plt.scatter(embedding[:, 0], embedding[:, 1], c=milestone_labels, cmap='tab20', s=1, alpha=0.3)
         #plt.show()
     if sc_labels_numeric is None:
         if via_object is not None:
             sc_labels_numeric = via_object.time_series_labels
-        else: print('please provide either a list of numeric labels (single cell level) or via_object')
+        else: print(f'{datetime.now()}\tWill use via-pseudotime for edges, otherwise consider providing a list of numeric labels (single cell level) or via_object')
     if sc_pt is None:
         sc_pt =via_object.single_cell_pt_markov
     '''
@@ -827,7 +889,7 @@ def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=
     #layout = vertex_milestone_graph.layout_fruchterman_reingold()
     #embedding = np.asarray(layout.coords)
 
-    print(f'{datetime.now()}\tmake node dataframe')
+    #print(f'{datetime.now()}\tmake node dataframe')
     data_node = [node for node in range(embedding.shape[0])]
     nodes = pd.DataFrame(data_node, columns=['id'])
     nodes.set_index('id', inplace=True)
@@ -859,7 +921,7 @@ def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=
     # seems to work better when allowing the bundling to occur on unweighted representation and later using length of segments to color code significance
     if weighted ==True: edges['weight'] = edges['weight0']#1  # [1/i for i in edges['weight0']]np.where((edges['source_cluster'] != edges['target_cluster']) , 1,0.1)#[1/i for i in edges['weight0']]#
     else: edges['weight'] = 1
-    print(f'{datetime.now()}\tHammer bundling')
+    print(f'{datetime.now()}\tMaking smooth edges')
 
 
     hb = hammer_bundle(nodes_mean, edges, weight='weight', initial_bandwidth=initial_bandwidth,
@@ -872,6 +934,7 @@ def make_edgebundle_milestone(embedding:ndarray=None, sc_graph=None, via_object=
     hb_dict['hammerbundle'] = hb
     hb_dict['milestone_embedding'] = nodes_mean
     hb_dict['edges'] = edges[['source','target']]
+    hb_dict['sc_milestone_labels'] = milestone_labels
     return hb_dict
 
 def make_edgebundle_sc(embedding, sc_graph, initial_bandwidth = 0.05, decay=0.70, sc_clusterlabel:list=[]):
@@ -957,7 +1020,10 @@ def infer_direction_piegraph(start_node, end_node, CSM, velocity_weight,pt, tanh
         print('csm', csm_se + -1 * csm_es)
         '''
         # Note csm_es does not equal -csm_se because the velocity vector used in the dot product refers to the originating cell
-    pt = pt * (3 / max(pt))  # bring the values to 0-3 so that the tanh function has a range of values
+
+    mpt=max(pt)
+    pt = [i*3 / mpt for i in pt]  # bring the values to 0-3 so that the tanh function has a range of values
+
     #print('pt of end node', end_node, round(pt[end_node], 2), 'pt of start node', start_node, round(pt[start_node], 2))
     tanh_ = math.tanh((pt[end_node] - pt[start_node]) * tanh_scaling_factor)
     #print('tanh', tanh_)
@@ -1080,8 +1146,8 @@ def stationary_probability_(A_velo):
 
     print(f"{datetime.now()}\tStationary distribution normed {np.round(pi,3)}")
     sorted_pi = np.argsort(pi)
-    velo_root_top3 = sorted_pi[0:3]
-    print(f"{datetime.now()}\tTop 3 candidates for root: {np.round(sorted_pi[0:3],2)} with stationary prob (%) {np.round(pi[sorted_pi[0:3]]*100,2)}")
+    velo_root_top3 = sorted_pi[0:10]
+    print(f"{datetime.now()}\tTop 5 candidates for root: {np.round(sorted_pi[0:10],2)} with stationary prob (%) {np.round(pi[sorted_pi[0:10]]*100,3)}")
     print(f"{datetime.now()}\tTop 5 candidates for terminal: {np.flip(sorted_pi)[0:5]}")
 
     return pi, velo_root_top3
@@ -1090,8 +1156,9 @@ def stationary_probability_(A_velo):
 
 def velocity_transition(A,V,G, slope =4):
     '''
-    Reweighting the cluster level transition matrix based on the cosine similarities of velocity vectors
-    relative to the change in gene expression from the ith cell to its neighbors in knn_gene graph (at cluster level)
+    Reweighting the cluster level transition matrix based on the cosine similarities of velocity vectors.
+    negative direction is suppresed using logistic function, and positive directions are emphasize using logistic function
+    positive direction is when velocity relative to the change in gene expression from the ith cell to its neighbors in knn_gene graph (at cluster level) is also positive, and converse
     :param A: Adjacency of clustergraph
     :param V: velocity matrix, cluster average
     :param G: Gene expression matrix, cluster average
@@ -1121,14 +1188,10 @@ def velocity_transition(A,V,G, slope =4):
     #CSM[mask] = 0 #remove non-neighbor edges
     A_velo[mask]=0
     A_velo = np.multiply(A_velo,A) #multiply element-wise the edge-weight of the transition matrix A by the velocity-factor
-    #print('A_velo col sum', np.sum(A_velo,axis=1))
-    print(f"{datetime.now()}\t Looking for initial states")
+
+    print(f"{datetime.now()}\tLooking for initial states")
     pi, velo_root_top3 = stationary_probability_(A_velo)
 
-    #print('Avelo')
-    #print(A_velo)
-    #print('CSM')
-    #print(CSM)
     return A_velo, CSM, velo_root_top3
 
 def sc_CSM(A, V, G):
@@ -1141,8 +1204,8 @@ def sc_CSM(A, V, G):
     CSM = np.zeros_like(A)
     find_A = find(A)
     size_A = A.size
-    time_0=  time.ctime()
-    #print('single-cell computation of sc_CSM')
+
+
     for i in range(A.shape[0]):
         delta_gene = G-G[i,:] #change in gene expression when going from i'th cell to other cells
         #print('shape delta_gene', delta_gene.shape, delta_gene)
@@ -1156,11 +1219,9 @@ def sc_CSM(A, V, G):
         #A_velo_i = np.insert(A_velo_i,obj=i,values=0)
         CSM_i = np.insert(CSM_i,obj=i,values=0)
         CSM[i,:] = CSM_i
-    #print('shape 0', CSM.shape)
-    #print('time taken non csr computation of sc_CSM', round(time.ctime()-time_0),2)
+
     CSM_list = []
-    time_1 = time.ctime()
-    #print('time start CSR computation of sc_CSM', time_1)
+
     for i in range(size_A):
         start = find_A[0][i]
         end = find_A[1][i]
@@ -1176,10 +1237,7 @@ def sc_CSM(A, V, G):
         # A_velo_i = np.insert(A_velo_i,obj=i,values=0)
     CSM = csr_matrix((CSM_list, (np.array(find_A[0]), np.array(find_A[1]))),
                        shape=size_A)
-    #print('shape 1', CSM.shape)
-    #print('CSM.data', CSM.data)
 
-    #print('time taken for CSR computation of sc_CSM', round(time.ctime() - time_1), 2)
     return CSM
 def interpolate_stream(array):
     from scipy import interpolate
@@ -1363,17 +1421,16 @@ def map_velocity_to_color(X1D, Y1D, U, V,segments):
     coords = np.vstack((X, Y)).T #(nx2) array of grid coords
     neigh_graph = NearestNeighbors(n_neighbors=1)
     neigh_graph.fit(coords)
-    print('number coords for grid', coords.shape)
-    print(np.isnan(coords))
+
+
 
     seg_coors = np.squeeze(segments[:, ::2, :])
-    print('seg coors', seg_coors.shape)
+
     neigh_seg = neigh_graph.kneighbors(seg_coors, return_distance=False)#[0]
-    print('neighbors of seg points', neigh_seg)
+
     n= len(segments)
     u_seg = velo_coords[neigh_seg,1]
 
-    print('u-seg', u_seg.shape)
     print(u_seg)
     C = np.zeros((n, 4))
     C[::-1] = 1-np.clip(u_seg,0.01,1)
@@ -1381,13 +1438,10 @@ def map_velocity_to_color(X1D, Y1D, U, V,segments):
     return C
 
 def interpolate_static_stream(x, y, u,v):
-    print('original U')
-    print(u)
+
     from scipy.interpolate import griddata
     x, y = np.meshgrid(x, y)
 
-    print('inside interpolate static stream')
-    print('mesh original', x.shape, y.shape)
     #print(x, y)
     points = np.array((x.flatten(), y.flatten())).T
     u = np.nan_to_num(u.flatten())
@@ -1395,16 +1449,11 @@ def interpolate_static_stream(x, y, u,v):
     xi = np.linspace(x.min(), x.max(), 25)
     yi = np.linspace(y.min(), y.max(), 25)
     X, Y = np.meshgrid(xi, yi)
-    #print('new mesh', X.shape, Y.shape)
-    #print('X new mesh')
-    #print(X)
-    #print('Y new mesh')
-    #print(Y)
+
 
     U = griddata(points, u, (X, Y), method='cubic')
     V = griddata(points, v, (X, Y), method='cubic')
-    print('interp static U', U.shape, U)
-    print('interp static V', V.shape, V)
+
     return X, Y, U, V
 
 def l2_norm(x: Union[ndarray, spmatrix], axis: int = 1) -> Union[float, ndarray]:
